from gpaw.tddft import *
from ase.structure import molecule
from ase import Atom, Atoms
from gpaw import GPAW, PoissonSolver, FermiDirac
from sys import argv
import numpy as np
from ase.units import Hartree, Bohr
from lcao_tddft import LCAOTDDFT
from ase.io import read
from gpaw.hgh import HGHParameterSet, HGHSetupData

hghsetup = HGHSetupData(HGHParameterSet('H', 1, 1.0, 0.3, [], []))

d = 3.4
atoms = read('au2_pbe_relax.xyz')
atoms = atoms[atoms.numbers != 1]
atoms.numbers[:] = 1
atoms.pbc = 1
#from ase.visualize import view
#atoms = Atoms('2Au', positions=([0,0,0], [0,0,d]))

from ase.md.velocitydistribution import MaxwellBoltzmannDistribution as MBD
from ase.units import kB
distr = MBD(atoms, 6000.0 * kB)

name = 'nacl'
atoms.center(vacuum=1)
#v0 = np.zeros((2,3))
#v0 = atoms.get_velocities()
#v0[0,2] = 0.2
#v0[1,2] = -0.2
#atoms.set_velocities(v0)
calc = LCAOTDDFT(atoms=atoms, nbands=int(len(atoms) * 0.7), #nbands=27,
                 setups={'H': hghsetup},
                 occupations=FermiDirac(0.1),
                 poissonsolver=PoissonSolver(relax='GS'),
                 mode='lcao', xc='oldLDA', h=0.2, basis='sz(dzp)', dtype=complex) #, kpts=[[0.001,0.002,0.003]])
#atoms.set_pbc((False, False, False))
atoms.set_calculator(calc)
E0 = atoms.get_potential_energy()
#F0 = atoms.get_forces()
calc.tddft_init()
E1 = calc.get_electronic_energy()
F1 = calc.get_forces()
print 'Energy from ASE: ', E0
print 'Energy from calc.hamiltonian: ', E1*Hartree
print 'positions =', atoms.get_positions()
print 'forces =', F1
#print 'Forces from ASE: ', F0
#calc2 = GPAW(h=0.2, xc='LDA')
#atoms.set_calculator(calc2)
#atoms.get_potential_energy()
#F_paw = atoms.get_forces()
#print 'Forces from calc.get_forces w/FD: ', F_paw
#print 'Forces from calc.get_forces w/LCAO: ', F1*Hartree/Bohr
#calc.absorption_kick([0.001,0,0])
time_step = 25.0 / 2.
niters = int(140e3/time_step)
#niters = 1
calc.propagate(time_step, niters, name+'_lcao_dzp_x.dm', 'x_nacl_dt10_eps2_g4p.dat', 'e_nacl_dt10_eps2_g4p.dat',
               sc_out = 'errsc_dt10_eps2_g4p.dat', ehrenfest=True, p_term=True, cn_predictor=False, 
               sc_eps=1e-7)
#photoabsorption_spectrum(name+'_lcao_qztp_x.dm', name+'_lcao_spectrum_qztp_x.dat')






