from ase.structure import molecule
from ase.optimize import QuasiNewton
from gpaw import GPAW
from ase.units import Bohr, AUT, _amu, _me, Hartree
from gpaw.external_potential import ConstantElectricField
import numpy as np
from math import sqrt
from gpaw.utilities.blas import gemm
from gpaw.utilities import unpack
from numpy.linalg import inv, eig
from numpy import dot, eye, array, asarray, zeros
from gpaw.mixer import DummyMixer
from math import pi
from ase.parallel import paropen
from gpaw.mpi import world
import time
from gpaw.tddft.units import attosec_to_autime

#de = open('debug.new','w')

class KickHamiltonian:
    def __init__(self, calc, ext):
        self.ext = ext
        self.vt_sG = [ ext.get_potential(gd=calc.density.gd) ]
        #print >> de, "vt_sG", self.vt_sG[0]
        self.dH_asp = {}

        # This code is copy-paste from hamiltonian.update
        for a, D_sp in calc.density.D_asp.items():
            print "Kick proc ", world.rank, " atom ", a
            setup = calc.hamiltonian.setups[a]
            vext = ext.get_taylor(spos_c=calc.hamiltonian.spos_ac[a, :])
            # Taylor expansion to the zeroth order
            self.dH_asp[a] = [ vext[0][0] * sqrt(4 * pi) * setup.Delta_pL[:, 0] ]
            if len(vext) > 1:
                # Taylor expansion to the first order
                Delta_p1 = np.array([setup.Delta_pL[:, 1],
                                     setup.Delta_pL[:, 2],
                                     setup.Delta_pL[:, 3]])
                self.dH_asp[a] += sqrt(4 * pi / 3) * np.dot(vext[1], Delta_p1)
        #print >>de, "dH_asp", self.dH_asp


class LCAOTDDFT(GPAW):
    def __init__(self, atoms=None, **kwargs):
        GPAW.__init__(self, **kwargs)
        self.kick_strength = [0.0, 0.0, 0.0]
        self.tddft_initialized = False
        #Ehrenfest
        if(atoms is not None):
            amu_to_aumass = _amu / _me
            self.atoms = atoms
            self.x = self.atoms.positions / Bohr
            #print 'self.x =', self.x
            self.v  = self.x.copy()
            #print atoms.get_velocities() is None
            if(self.atoms.get_velocities() is None):
                self.v[:] = 0.0
                self.atoms.set_velocities(self.v)
            else:
                self.v = self.atoms.get_velocities() / (Bohr / AUT)
            self.M = self.atoms.get_masses() * amu_to_aumass
            self.a  = self.v.copy()
            #self.ah = self.v.copy()
            self.an = self.v.copy()
            #self.vh = self.v.copy()
            #self.xh = self.v.copy()
            #self.vt = self.v.copy()
            self.F = self.v.copy()
            #for i in range(len(self.F)):	            
            #    self.a[i] = self.F[i] / self.M[i]
            self.a[:] = 0.0
            

    #Bohr-Oppenheimer MD for debugging purposes
    def bo_dynamics(self, dt, max_steps, x_out = 'coords_bo.dat',
                    e_out = 'energies_bo.dat'):
        dt *= attosec_to_autime
        self.time = 0
        steps = 0
        self.x_file = paropen(x_out, 'w')
        self.e_file = paropen(e_out, 'w')
        
        while 1:
            if(self.time == 0):
                self.F = self.get_forces()
            self.x_file.write('%5.6f %5.6f %5.6f\n' %(self.time/attosec_to_autime, self.x[0,2]*Bohr
                                                      , self.x[1,2]*Bohr))
            #print >>self.x_file, line_x
            #update PAW calculation
            #self.atoms.set_calculator(self)
            #self.atoms.get_potential_energy()
            self.initialize()
            self.calculate(self.atoms, converge=True, force_call_to_set_positions=True)
            self.get_energy(bo=True)
            #Epot = self.atoms.get_potential_energy() / Hartree
            self.e_file.write('%5.6f %5.8f %5.8f %5.8f\n' % (self.time/attosec_to_autime, self.Ekin*Hartree, 
                                                                   self.Epot*Hartree, self.Etot*Hartree))
            #print >>self.e_file, line_e
            self.x_file.flush()
            self.e_file.flush()
            #print 'self.x =', self.x
            #print 'atoms.pos =',self.atoms.positions/Bohr
            #print 'self.v =', self.v
            #print 'self.a =', self.a
                
            for i in range(len(self.F)):	            
                self.a[i] = self.F[i] / self.M[i]
            self.x = self.x + self.v * dt + .5 * self.a * dt**2
            self.update_positions()
            self.forces.reset()
            #self.update_projectors()
            F_tmp = self.get_forces()
            #a(t+dt)
            for i in range(len(self.F)):	            
                self.an[i] = F_tmp[i]/ self.M[i]
            #v = v + .5*(a(t) + a(t+dt))*dt
            self.v = self.v + .5 * (self.a + self.an) * dt
            #update F and a to t+dt
            self.F[:] = F_tmp
            self.a[:] = self.an[:]
            self.time += dt
            steps += 1
            if(steps > max_steps):
                break
        

    def absorption_kick(self, strength):
        self.tddft_init()
        self.kick_strength = strength

        # magnitude
        magnitude = np.sqrt(strength[0]*strength[0] 
                             + strength[1]*strength[1] 
                             + strength[2]*strength[2])

        # normalize
        direction = strength / magnitude

        if world.rank == 0:
            print "Applying absorbtion kick"
            print "Magnitude: ", magnitude
            print "Direction: ", direction

        print "gamma", self.wfs.basis_functions.gamma

        # Create hamiltonian object for absorbtion kick
        kick_hamiltonian = KickHamiltonian(self, ConstantElectricField(magnitude, direction=direction))
        
        Vkick_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(kick_hamiltonian, self.wfs, self.wfs.kpt_u[0], add_kinetic=False)
        print "V_KICK_MM", Vkick_MM
        #print >>de, "Vkick_MM", Vkick_MM
   
        # Apply kick
        Ukick_MM = dot(inv(self.S_MM-0.5j*0.1*Vkick_MM), self.S_MM+0.5j*0.1*Vkick_MM)
        for i in range(0,10):
            self.C_nM = dot(self.C_nM, Ukick_MM.T)

        
    def tddft_init(self):
        if not self.tddft_initialized:
             self.density.mixer = DummyMixer()    # Reset the density mixer
             self.S_MM = self.wfs.S_qMM[0]        # Get the overlap matrix
             #print >> de, "initS", self.S_MM
             self.C_nM = self.wfs.kpt_u[0].C_nM.copy()   # Copy the LCAO matrix
             #print >> de, "initC", self.C_nM
             #reserve memory for the P matrix
             self.P_MM = self.wfs.S_qMM[0].copy()
             assert self.P_MM.dtype == complex
             self.P_MM[:] = 0.0
             #Convergence criterion for TD-SCF loop
             self.sc_eps = 1e-5
             #Construct the inverse mapping a_M
             ksl = self.wfs.ksl
             nM = ksl.nao
             #print 'nM =', nM
             basis_functions = self.wfs.basis_functions
             self.a_M = []
             la = len(basis_functions.my_atom_indices)
             for a in basis_functions.my_atom_indices:
                 M_a = basis_functions.M_a[a]
                 if(a < (la-1)):
                     M_a_next = basis_functions.M_a[a+1]
                 else:
                     M_a_next = nM
                 for m in range(M_a, M_a_next):
                     self.a_M.append(a)

             self.tddft_initialized = True

    def update_positions(self):
        self.atoms.positions = self.x * Bohr
        self.set_positions(self.atoms)
        #self.forces.reset()
        #self.hamiltonian.set_positions(x_a)
        #self.density.set_positions(x_a)

    def get_electronic_energy(self):
        #self.overlap.update(self.wfs)
        #Update kpt.eps_n
        #These updates change something, XXX
        #for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
        #    P_ni.fill(117)
        #    gemm(1.0, self.wfs.kpt_u[0].P_aMi[a], self.wfs.kpt_u[0].C_nM, 0.0, P_ni, 'n')
        #Vt_xMM = self.wfs.basis_functions.calculate_potential_matrices(self.hamiltonian.vt_sG[0])
        H_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(self.hamiltonian, 
                                                                 self.wfs, self.wfs.kpt_u[0])
        C_nM = self.wfs.kpt_u[0].C_nM
        for kpt in self.wfs.kpt_u:
            nb = C_nM.shape[0]
            for n in range(nb):
                C_M = C_nM[n].T
                HC = np.dot(H_MM, C_M)
                eps_tmp = np.dot(C_M.conj(), HC)
                kpt.eps_n[n] = eps_tmp.real
        self.occupations.calculate_band_energy(self.wfs)
	
        return self.hamiltonian.get_energy(self.occupations)
    
    def get_electronic_bo(self):
        return self.hamiltonian.get_energy(self.occupations)

    #Current implementation: BO forces
    def get_forces(self):
        #self.update_projectors()
        #self.density.update(self.wfs)
        #self.hamiltonian.update(self.density)
        #self.forces.reset()
        return self.forces.calculate(self.wfs, self.density, self.hamiltonian)

    #Calculates the correction terms required for Ehrenfest dynamics
    #OLD, SLOW version
    def calculate_td_correction(self):
        #M = self.P_MM.shape[0] #number of LCAO functions
        tci = self.wfs.tci
        ksl = self.wfs.ksl
        kd = self.wfs.kd
        nao = ksl.nao
        nq = len(kd.ibzk_qc)
        mynao = ksl.mynao
        dtype = self.wfs.dtype
        basis_functions = self.wfs.basis_functions
        setups = self.wfs.setups
        #Calculate some integrals first
        spos_ac = tci.atoms.get_scaled_positions() % 1.0
        dThetadR_qvMM = np.empty((nq, 3, mynao, nao), dtype)
        dTdR_qvMM = np.empty((nq, 3, mynao, nao), dtype)
        dPdR_aqvMi = {}
        #O_aii = {}
        #Delta_aivv = {}
        v_atom = self.v
        #print 'v_atom =', self.v
        P_aqMi = self.wfs.P_aqMi
        #O and Delta from self.setups[a]
        for a in basis_functions.my_atom_indices:
            ni = setups[a].ni
            dPdR_aqvMi[a] = np.empty((nq, 3, nao, ni), dtype)
            #t0 = time.clock()
            tci.calculate_derivative(spos_ac, dThetadR_qvMM, dTdR_qvMM,dPdR_aqvMi)
            #print 'TCI derivatives took', time.clock()-t0, 'seconds'
        P1nm_sum = 0.0
        #print 'max of dT/dR = ', dThetadR_qvMM.max()
        #print 'min of dT/dR = ', dThetadR_qvMM.min()
        print 'Calculating P matrix'
        t0 = time.clock()
        #Zero the P matrix first
        self.P_MM[:] = 0.0
        for nu in range(nao):
            for mu in range(nao):
                a_mu = self.a_M[mu]
                #P2nm_sum = 0.0
                for c in range(3):
                    P2nm_sum = 0.0
                    #<\Phi_nu | d / dRa_c \Phi_mu>
                    dThetadR_c = dThetadR_qvMM[0,c,nu,mu]
                    P2nm_sum += dThetadR_c
                    for a in basis_functions.my_atom_indices:
                        O_ii = setups[a].dO_ii
                        P_Mi = P_aqMi[a][0,:,:]
                        #<d / dRa_c p_i^a | \Phi_mu>
                        dPdR_i = dPdR_aqvMi[a][0,c,mu,:]
                        #<p_i^a | d/dRa_c \Phi_mu>
                        dThetadR_i = -dPdR_i
                        v_c = v_atom[a,c]
                        #nabla_iiv = <phi_i | d/dr_v phi_j> - <phit_i | d/dr_v phit_j>
                        #Note that <phi_i | d/dr phi_j> = - <phi_i | d/dRa Phi_j>
                        if setups[a].nabla_iiv is not None:
                            dphidR_ii = -setups[a].nabla_iiv[:,:,c]
                            OdPdR = np.dot(O_ii, dPdR_i.T)
                            dphidRP = np.dot(dphidR_ii, P_Mi[mu,:].T)
                            P1nm_sum = np.dot(P_Mi[nu,:].T, dphidRP+OdPdR)
                            OdThetadR = np.dot(O_ii, dThetadR_i.T)
                            P2nm_sum += np.dot(P_Mi[nu,:].T, OdThetadR)

                            #Add P(1)
                            self.P_MM[nu,mu] += -1j * v_c * P1nm_sum
                    #Add P(2)
                    self.P_MM[nu,mu] += -1j * v_atom[a_mu,c] * P2nm_sum
        self.P_MM[:] = 0.0

    #NEW implementation of the P matrix
    def calculate_td_correction2(self):
        #M = self.P_MM.shape[0] #number of LCAO functions
        print 'Calculating P matrix'
        t0 = time.clock()
        tci = self.wfs.tci
        ksl = self.wfs.ksl
        kd = self.wfs.kd
        nao = ksl.nao
        nq = len(kd.ibzk_qc)
        mynao = ksl.mynao
        dtype = self.wfs.dtype
        basis_functions = self.wfs.basis_functions
        setups = self.wfs.setups
        #Calculate some integrals first
        spos_ac = tci.atoms.get_scaled_positions() % 1.0
        dThetadR_qvMM = np.empty((nq, 3, mynao, nao), dtype)
        dTdR_qvMM = np.empty((nq, 3, mynao, nao), dtype)
        dPdR_aqvMi = {}
        #O_aii = {}
        #Delta_aivv = {}
        v_atom = self.v
        #print 'v_atom =', self.v
        P_aqMi = self.wfs.P_aqMi
        #O and Delta from self.setups[a]
        for a in basis_functions.my_atom_indices:
            ni = setups[a].ni
            dPdR_aqvMi[a] = np.empty((nq, 3, nao, ni), dtype)
            #t0 = time.clock()
        tci.calculate_derivative(spos_ac, dThetadR_qvMM, dTdR_qvMM,dPdR_aqvMi)
            #print 'TCI derivatives took', time.clock()-t0, 'seconds'
        P1nm_sum = 0.0
        #print 'max of dT/dR = ', dThetadR_qvMM.max()
        #print 'min of dT/dR = ', dThetadR_qvMM.min()
        #Zero the P matrix first
        self.P_MM[:] = 0.0
        # askhl XXX P2_cMM = np.array([self.P_MM.copy(), self.P_MM.copy(), self.P_MM.copy()])
        # Okay, here we have the error.  Let us recalculate P from scratch.

        bfs = self.wfs.basis_functions
        Mstart = 0
        def _slices(indices):
            for a in indices:
                M1 = bfs.M_a[a] - Mstart
                M2 = M1 + self.wfs.setups[a].nao
                if M2 > 0:
                    yield a, max(0, M1), M2

        def slices():
            return _slices(bfs.atom_indices)

        def my_slices():
            return _slices(bfs.my_atom_indices)


        for a, M1, M2 in my_slices():
            #if bfs.M_a[a] == M1:
            #    continue
            dThetadR_dRdt_Mm = (dThetadR_qvMM[0, :, :, M1:M2]
                                * v_atom[a, :, None, None]).sum(axis=0)
            self.P_MM[:, M1:M2] = -1j * dThetadR_dRdt_Mm

        if 0: # askhl XXX
            for a in basis_functions.my_atom_indices:
                if setups[a].nabla_iiv is not None:
                    P_Mi = P_aqMi[a][0,:,:]
                    O_ii = setups[a].dO_ii
                    for c in range(3):
                        dphidR_iic = -setups[a].nabla_iiv[:,:,c]
                        dPdR_c = dPdR_aqvMi[a][0,c,:,:]
                        OdPdR = np.dot(O_ii, dPdR_c.T)
                        dphidRP = np.dot(dphidR_iic, P_Mi.T)
                        P1 = np.dot(P_Mi, dphidRP+OdPdR)
                        v_c = v_atom[a,c]
                        #Add P(1)
                        self.P_MM += -1j * v_c * P1
                        #Calculate P2_MM
                        #\sum_{a,i,j} <\Phi_nu | p_i^a> O^a_ij <p_j^a | d/dRamu \Phi_mu>
                        P2_cMM[c] += np.dot(P_Mi, -OdPdR)

        #Add -i R_amu * <\Phi_nu | d/dRa \Phi_mu>
        if 0: # askhl XXX 
            for mu in range(nao):
                a_mu = self.a_M[mu]
                for c in range(3):
                    dThetadR_cmu = dThetadR_qvMM[0,c,:,mu]
                    self.P_MM[:,mu] += -1j * v_atom[a_mu,c] * dThetadR_cmu
                    self.P_MM[:,mu] += -1j * v_atom[a_mu,c] * P2_cMM[c,:,mu]    
        
        print 'Finished calculating P matrix'
        print 'Evaluation the P term took', time.clock()-t0, 'seconds'
        #print self.P_MM.max()
                                

        
            
        #print dPdR_aqvMi[0].min()
        #print basis_functions.my_atom_indices
        #Simple and stupid, just loop over everything
        #for nu in range(M):
        #    for mu in range(M):       

    def get_energy(self,bo=False):
        """Updates kinetic, electronic and total energies"""
        self.Ekin = 0.0
        for i in range(len(self.v)):
            self.Ekin += (
                .5 * self.M[i] * self.v[i][0]**2
                 +.5 * self.M[i] * self.v[i][1]**2
                 +.5 * self.M[i] * self.v[i][2]**2
                 )
        if(not bo):
            self.Epot = self.get_electronic_energy()
        else:
            self.Epot = self.get_electronic_bo()
        self.Etot = self.Ekin + self.Epot

    def update_projectors(self):
        self.timer.start('LCAO update projectors') 
        # Update projectors, (is this needed?)
        for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
            P_ni.fill(117)
            gemm(1.0, self.wfs.P_aqMi[a][0], self.C_nM, 0.0, P_ni, 'n')
                #print >>de, "P_ni", P_ni
        self.timer.stop('LCAO update projectors') 
        self.wfs.kpt_u[0].C_nM[:] = self.C_nM
        self.density.update(self.wfs)
        self.hamiltonian.update(self.density)

    def propagate(self, dt, max_steps, out='lcao.dm', x_out='coords.dat', e_out='energies.dat',
                  sc_out = 'sc_err.dat', ehrenfest=True, p_term=True, cn_predictor=True, 
                  sc_eps=1e-5):
        dt *= attosec_to_autime
        self.dm_file = paropen(out,'w')
        self.energy_file = paropen('td_energy.dat', 'w')
        self.sc_eps = sc_eps
        if(ehrenfest):
            self.x_file = paropen(x_out, 'w')
            self.e_file = paropen(e_out, 'w')
            self.sc_file = paropen(sc_out, 'w')
        #self.tddft_init()

        header = '# Kick = [%22.12le, %22.12le, %22.12le]\n' \
               % (self.kick_strength[0], self.kick_strength[1], \
                  self.kick_strength[2])
        header += '# %15s %15s %22s %22s %22s\n' \
               % ('time', 'norm', 'dmx', 'dmy', 'dmz')
        self.dm_file.write(header)
        self.dm_file.flush()

        dm0 = None # Initial dipole moment
        self.time = 0
        steps = 0
        H_MM_old2 = self.S_MM.copy()
        first_iter = True
        while 1:
            if(not ehrenfest):
                dm = self.density.finegd.calculate_dipole_moment(self.density.rhot_g)
                if dm0 is None:
                    dm0 = dm
                norm = self.density.finegd.integrate(self.density.rhot_g)
                line = '%20.8lf %20.8le %22.12le %22.12le %22.12le\n' % (self.time, norm, dm[0], dm[1], dm[2])
                print >>self.dm_file, line,
                print line,
                self.dm_file.flush()

            self.timer.start('LCAO update projectors') 
            # Update projectors, (is this needed?)
            #for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
            #    P_ni.fill(117)
            #    gemm(1.0, self.wfs.kpt_u[0].P_aMi[a], self.C_nM, 0.0, P_ni, 'n')
                #print >>de, "P_ni", P_ni
            # Update projectors, (is this needed?)
            self.timer.stop('LCAO update projectors') 
            #self.wfs.kpt_u[0].C_nM[:] = self.C_nM
            t0 = time.clock()
            #Update C
            self.C_nM[:] = self.wfs.kpt_u[0].C_nM[:]
            #Update positions
            self.atoms.positions = self.x * Bohr
            self.set_positions(self.atoms)
            self.density.update(self.wfs)
            #self.hamiltonian.update(self.density)
            #Update projectors, is this needed?
            for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
                P_ni.fill(117)
                gemm(1.0, self.wfs.P_aqMi[a][0], self.C_nM, 0.0, P_ni, 'n')
            self.forces.reset()
            #F(R(t), rho(t))
            self.F = self.get_forces()

            for i in range(len(self.F)):
                self.a[i] = self.F[i]/self.M[i]

            print 'Force evaluation 1+updates took', time.clock()-t0, 'seconds'

            #Calculate TD energy
            #E_td = self.get_electronic_energy()
            #self.energy_file.write('%5.6f %5.10f\n' % (self.time, E_td))
            #self.energy_file.flush()
            if(ehrenfest):
                self.x_file.write('%5.6f %5.6f %5.6f\n' %(self.time/attosec_to_autime, self.x[0,2]*Bohr, 
                                                          self.x[1,2]*Bohr))
                #print >>self.x_file, line_x
                self.get_energy()
                self.e_file.write('%5.6f %5.8f %5.8f %5.8f\n' % (self.time/attosec_to_autime, self.Ekin*Hartree, 
                                                                 self.Epot*Hartree, self.Etot*Hartree))
                #print >>self.e_file, line_e
                self.x_file.flush()
                self.e_file.flush()

            #tempC_nM = self.C_nM.copy()
            #self.F = self.get_forces()
                #print 'predictor forces =', self.F
                #print 'positions before Verlet', self.x
            #    for i in range(len(self.F)):	            
            #        self.a[i] = self.F[i] / self.M[i]
            #    self.x = self.x + self.v * dt + .5 * self.a * dt**2
            #    self.update_positions(self.x)
            #    self.update_projectors()    
            #Testing
            if(p_term):
                #self.calculate_td_correction()
                self.calculate_td_correction2()


            if 0:
                #H(t+dt)
                t0 = time.clock()
                self.timer.start('Propagate')
                H_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(self.hamiltonian, self.wfs, self.wfs.kpt_u[0])
                #Symmetrize
                H_MM = .5 * (H_MM + H_MM.T)
                H_MM_old = H_MM.copy()
                S_MM = self.S_MM
                Sinv = np.linalg.inv(S_MM)
                M = len(S_MM)
                I = np.eye(M)
                nb = self.wfs.bd.nbands
                #Crank-Nicholson predictor
                lhs_MM = S_MM + .5j * dt * (H_MM+self.P_MM)
                for n in range(nb):
                #lhs_MM = np.eye(M) + .5j * dt * np.dot
                #lhs_MM = self.S_MM + .5j * dt * (H_MM+self.P_MM)
                    rhs_M = dot(S_MM - .5j * dt * (H_MM+self.P_MM), self.C_nM[n,:].T)
                    self.wfs.kpt_u[0].C_nM[n,:] = np.linalg.solve(lhs_MM, rhs_M).T

                #self.wfs.kpt_u[0].C_nM[:] = self.C_nM[:]
                #self.density.update(self.wfs)
                #self.hamiltonian.update(self.density)

                #Crank-Nicholson TD-SCF loop
                #norm(H) used as the SCF parameter, could be something else
                H_MM_prev = H_MM_old.copy()
                print '||H|| =', np.linalg.norm(H_MM_prev)
                for k in range(20):
                    self.density.update(self.wfs)
                    self.hamiltonian.update(self.density)
                    #Update projectors - is this needed?
                    for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
                        P_ni.fill(117)
                        gemm(1.0, self.wfs.P_aqMi[a][0], self.wfs.kpt_u[0].C_nM, 0.0, P_ni, 'n')
                    H_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(self.hamiltonian, self.wfs, self.wfs.kpt_u[0])
                    #Symmetrize
                    #H_half = 1/2 (H(t) + H_pred (t+dt))
                    H_MM = .5 * (H_MM_prev + H_MM)
                    sc_err = np.linalg.norm(H_MM-H_MM_prev)/np.linalg.norm(H_MM_prev)
                    #print 'corrector iterations completed:', k
                    #print 'sc_error =', sc_err
                    if(sc_err < self.sc_eps):
                        #print 'sc_iters =', k
                        self.sc_file.write('%3.6f %d %3.10f\n' % (self.time/attosec_to_autime, k, sc_err))
                        self.sc_file.flush()
                        break
                    H_MM_prev = H_MM.copy()
                    # left hand side S + .5i * dt * (H+P)
                    lhs_MM = S_MM + .5j * dt * (H_MM + self.P_MM)
                    for n in range(nb):
                        #Right hand side S - .5i * dt * (H+P)
                        rhs_M = dot(S_MM - .5j * dt * (H_MM + self.P_MM), self.C_nM[n,:].T)
                        self.wfs.kpt_u[0].C_nM[n,:] = np.linalg.solve(lhs_MM, rhs_M).T

                #self.wfs.kpt_u[0].C_nM[:] = self.C_nM[:]

                        #for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
                        #    P_ni.fill(117)
                        #    gemm(1.0, self.wfs.kpt_u[0].P_aMi[a], self.wfs.kpt_u[0].C_nM, 0.0, P_ni, 'n')

                self.timer.stop('Propagate')
                print 'Time propagation took', time.clock()-t0, 'seconds'

            else:
                #XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


                # Update projectors, (is this needed?)
                t0 = time.clock()
                self.timer.start('Propagate')

                #print 'band energy =', self.occupations.e_band
                #Test CN
                #Hamiltonian matrix H(t)
                #for a in self.wfs.basis_functions.my_atom_indices:
                #    print 'a =', a
                #    print 'M corresponding to a is =', self.wfs.basis_functions.M_a[a]
                #print 'self.a_M =', self.a_M

                #if(debug):
                #    self.P_MM[:] = 0.0

                #self.forces.reset()
                #F_old = self.get_forces()
                M = self.wfs.ksl.nao
                nb = self.C_nM.shape[0]
                S_MM = self.wfs.kpt_u[0].S_MM
                #Symmetrize
                S_MM = .5 * (S_MM + S_MM.T)

                #H(t+dt)
                t0 = time.clock()
                H_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(self.hamiltonian, self.wfs, self.wfs.kpt_u[0])
                #Symmetrize
                H_MM = .5 * (H_MM + H_MM.T)
                H_MM_old = H_MM.copy()
                c1 = .5 - np.sqrt(3)/6
                c2 = .5 + np.sqrt(3)/6
                Sinv = np.linalg.inv(S_MM)
                I = np.eye(M)
                if(cn_predictor):
                    #Crank-Nicholson predictor
                    lhs_MM = S_MM + .5j * dt * (H_MM+self.P_MM)
                    for n in range(nb):
                    #lhs_MM = np.eye(M) + .5j * dt * np.dot
                    #lhs_MM = self.S_MM + .5j * dt * (H_MM+self.P_MM)
                        rhs_M = dot(S_MM - .5j * dt * (H_MM+self.P_MM), self.C_nM[n,:].T)
                        self.wfs.kpt_u[0].C_nM[n,:] = np.linalg.solve(lhs_MM, rhs_M).T

                else:
                    #Gauss 4 predictor

                #if(self.time == 0):
                #    H1 = a2 * t1**2 + a1 * t1 + a0
                #    H2 = a2 * t2**2 + a1 * t2 + a0

                    G = -1j * np.dot(Sinv, H_MM + self.P_MM)
                #G2 = -1j * np.dot(Sinv, H_MM + self.P_MM)
                #M changes
                    Mp = I -np.sqrt(3)/6 * dt * G
                    Minv = np.linalg.inv(Mp)
                    MinvG = np.dot(Minv, G)
                    A = dt/2 * MinvG * (.5 - 1/np.sqrt(3))\
                        + dt/4 * G + np.sqrt(3)/12 * dt * dt * np.dot(G, MinvG)\
                        * (.5 - 1/np.sqrt(3))
                    B = dt/2 * MinvG * (.5 + 1/np.sqrt(3))\
                        + dt/4 * G + np.sqrt(3)/12 * dt * dt * np.dot(G, MinvG)\
                        * (.5 + 1/np.sqrt(3))
                    lhs_MM = I-A
                    for n in range(nb):
                    #lhs_MM = np.eye(M) + .5j * dt * np.dot
                    #lhs_MM = self.S_MM + .5j * dt * (H_MM+self.P_MM)
                    #rhs_M = dot(self.S_MM - .5j * dt * (H_MM+self.P_MM), self.C_nM[n,:].T)
                        rhs_M = dot(I+B, self.C_nM[n,:].T)
                        self.wfs.kpt_u[0].C_nM[n,:] = np.linalg.solve(lhs_MM, rhs_M).T

                #self.wfs.kpt_u[0].C_nM[:] = self.C_nM[:]
                #self.density.update(self.wfs)
                #self.hamiltonian.update(self.density)

                #Gauss 4 TD-SCF loop
                #norm(H) used as the SCF parameter, could be something else
                H_MM_prev = H_MM_old.copy()
                print '||H|| =', np.linalg.norm(H_MM_prev)
                for k in range(20):
                    self.density.update(self.wfs)
                    self.hamiltonian.update(self.density)
                    #Update projectors - is this needed?
                    for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
                        P_ni.fill(117)
                        gemm(1.0, self.wfs.P_aqMi[a][0], self.wfs.kpt_u[0].C_nM, 0.0, P_ni, 'n')
                    H_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(self.hamiltonian, self.wfs, self.wfs.kpt_u[0])
                    #Symmetrize
                    H_MM = .5 * (H_MM + H_MM.T)
                    sc_err = np.linalg.norm(H_MM-H_MM_prev)/np.linalg.norm(H_MM_prev)
                    #print 'corrector iterations completed:', k
                    #print 'sc_error =', sc_err
                    if(sc_err < self.sc_eps):
                        #print 'sc_iters =', k
                        self.sc_file.write('%3.6f %d %3.10f\n' % (self.time/attosec_to_autime, k, sc_err))
                        self.sc_file.flush()
                        break
                    H_MM_prev = H_MM.copy()
                    H1 = (1-c1) * H_MM_old + c1 * H_MM
                    H2 = (1-c2) * H_MM_old + c2 * H_MM

                    G1 = -1j * np.dot(Sinv, H1 + self.P_MM)
                    G2 = -1j * np.dot(Sinv, H2 + self.P_MM)
                    Mp = I -np.sqrt(3)/6 * dt * G1
                    Minv = np.linalg.inv(Mp)
                    MinvG1 = np.dot(Minv, G1)
                    A = dt/2 * MinvG1 * (.5 - 1/np.sqrt(3))\
                        + dt/4 * G2 + np.sqrt(3)/12 * dt * dt * np.dot(G2, MinvG1)\
                        * (.5 - 1/np.sqrt(3))
                    B = dt/2 * MinvG1 * (.5 + 1/np.sqrt(3))\
                        + dt/4 * G2 + np.sqrt(3)/12 * dt * dt * np.dot(G2, MinvG1)\
                        * (.5 + 1/np.sqrt(3))
                    lhs_MM = I-A
                    for n in range(nb):
                    #lhs_MM = np.eye(M) + .5j * dt * np.dot
                    #lhs_MM = self.S_MM + .5j * dt * (H_MM+self.P_MM)
                    #rhs_M = dot(self.S_MM - .5j * dt * (H_MM+self.P_MM), self.C_nM[n,:].T)
                        rhs_M = dot(I+B, self.C_nM[n,:].T)
                        self.wfs.kpt_u[0].C_nM[n,:] = np.linalg.solve(lhs_MM, rhs_M).T

                #self.wfs.kpt_u[0].C_nM[:] = self.C_nM[:]

                        #for a, P_ni in self.wfs.kpt_u[0].P_ani.items():
                        #    P_ni.fill(117)
                        #    gemm(1.0, self.wfs.kpt_u[0].P_aMi[a], self.wfs.kpt_u[0].C_nM, 0.0, P_ni, 'n')

                self.timer.stop('Propagate')
                print 'Time propagation took', time.clock()-t0, 'seconds'
            t0 = time.clock()
            #Velocity Verlet 1
            steps += 1
            # Update projectors, (is this needed?)
            #self.set_positions(self.atoms)
            self.forces.reset()
            #F(R(t), rho(t+dt))
            self.F = self.get_forces()
            for i in range(len(self.F)):
                self.an[i] = self.F[i] / self.M[i]
            self.x  = self.x + self.v * dt + .5 * self.a * dt * dt	
            # Update projectors, (is this needed?)

            #v(t+dt) = v(t) + .5 * dt * (a(R(t),rho(t)) + a(R(t),rho(t+dt)))
            self.v = self.v + .5 * dt * (self.a + self.an)
            #self.set_positions(self.atoms)
            print 'Force evaluation 2 took', time.clock()-t0, 'seconds'
            #self.C_nM = dot(self.C_nM, U_MM.T)
            #self.a[:] = self.an
            #self.atoms.positions = self.x * Bohr
            self.time += dt
            
            if steps > max_steps:
                break
        self.dm_file.close()

"""
np.set_printoptions(threshold=np.nan)
mol = 'H2'
atoms = molecule(mol)
atoms.set_pbc((False, False, False))
#atoms.set_pbc((True,True,True))
atoms.center(vacuum=6)
calc = LCAOTDDFT(xc='LDA', h=0.3, # gpts=(40,40,24),
            mode='lcao', 
            basis='sz', 
            dtype=complex,
            convergence = {'energy': 0.0005,  
                           'density': 1.0e-6,
                           'eigenstates': 4.0e-10})
atoms.set_calculator(calc)

if False:
    relax = QuasiNewton(atoms)
    relax.run()
else:
    atoms.get_potential_energy()

calc.absorption_kick([0.0,0.0,0.001])
calc.propagate(8, 1000)
calc.write(mol+'.gpw',mode='all')

"""
